# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing
from json.decoder import JSONDecodeError

from .......core.api_error import ApiError
from .......core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from .......core.datetime_utils import serialize_datetime
from .......core.http_response import AsyncHttpResponse, HttpResponse
from .......core.jsonable_encoder import jsonable_encoder
from .......core.pydantic_utilities import parse_obj_as
from .......core.request_options import RequestOptions
from ....common.errors.bad_request_error import BadRequestError
from ....common.errors.not_found_error import NotFoundError
from ....common.errors.version_conflict_error import VersionConflictError
from ....common.types.coverage_id import CoverageId
from ....common.types.error_base_4_xx import ErrorBase4Xx
from ....common.types.page_token import PageToken
from ....common.types.payer_plan_group_id import PayerPlanGroupId
from ....common.types.sort_direction import SortDirection
from ....common.types.version_conflict_error_body import VersionConflictErrorBody
from ....eligibility_checks.resources.v_1.types.eligibility_check_metadata import EligibilityCheckMetadata
from .types.coverage import Coverage
from .types.coverage_eligibility_check_response import CoverageEligibilityCheckResponse
from .types.coverages_page import CoveragesPage
from .types.mutable_coverage import MutableCoverage
from .types.payer_plan_group_fields import PayerPlanGroupFields
from .types.service_type_code import ServiceTypeCode

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawV1Client:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def create(
        self, *, request: MutableCoverage, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[Coverage]:
        """
        Creates a new Coverage. A Coverage provides the high-level identifiers and descriptors of a specific insurance plan for a specific individual - typically the information you can find on an insurance card. Additionally a coverage will include detailed benefits information covered by the specific plan for the individual.

        Parameters
        ----------
        request : MutableCoverage

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[Coverage]
        """
        _response = self._client_wrapper.httpx_client.request(
            "coverages/v1",
            base_url=self._client_wrapper.get_environment().pre_encounter,
            method="POST",
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                Coverage,
                parse_obj_as(
                    type_=Coverage,  # type: ignore
                    object_=_response_json,
                ),
            )
            return HttpResponse(response=_response, data=_data)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def update(
        self,
        id: CoverageId,
        version: str,
        *,
        request: MutableCoverage,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[Coverage]:
        """
        Updates a Coverage. The path must contain the next version number to prevent race conditions. For example, if the current version of the coverage is n, you will need to send a request to this endpoint with `/{id}/n+1` to update the coverage. Updating historic versions is not supported.

        Parameters
        ----------
        id : CoverageId

        version : str

        request : MutableCoverage

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[Coverage]
        """
        _response = self._client_wrapper.httpx_client.request(
            f"coverages/v1/{jsonable_encoder(id)}/{jsonable_encoder(version)}",
            base_url=self._client_wrapper.get_environment().pre_encounter,
            method="PUT",
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                Coverage,
                parse_obj_as(
                    type_=Coverage,  # type: ignore
                    object_=_response_json,
                ),
            )
            return HttpResponse(response=_response, data=_data)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "NotFoundError":
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorBase4Xx,
                        parse_obj_as(
                            type_=ErrorBase4Xx,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "VersionConflictError":
                raise VersionConflictError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        VersionConflictErrorBody,
                        parse_obj_as(
                            type_=VersionConflictErrorBody,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get_multi_paginated(
        self,
        *,
        patient_id: typing.Optional[str] = None,
        payer_plan_group_id: typing.Optional[str] = None,
        page_token: typing.Optional[PageToken] = None,
        limit: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[CoveragesPage]:
        """
        Returns a page of Coverages based on the search criteria.

        Parameters
        ----------
        patient_id : typing.Optional[str]

        payer_plan_group_id : typing.Optional[str]

        page_token : typing.Optional[PageToken]

        limit : typing.Optional[int]
            Must be between 0 and 1000. Defaults to 100

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[CoveragesPage]
        """
        _response = self._client_wrapper.httpx_client.request(
            "coverages/v1/get-multi-paginated",
            base_url=self._client_wrapper.get_environment().pre_encounter,
            method="GET",
            params={
                "patient_id": patient_id,
                "payer_plan_group_id": payer_plan_group_id,
                "page_token": page_token,
                "limit": limit,
            },
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                CoveragesPage,
                parse_obj_as(
                    type_=CoveragesPage,  # type: ignore
                    object_=_response_json,
                ),
            )
            return HttpResponse(response=_response, data=_data)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "BadRequestError":
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorBase4Xx,
                        parse_obj_as(
                            type_=ErrorBase4Xx,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get(self, id: CoverageId, *, request_options: typing.Optional[RequestOptions] = None) -> HttpResponse[Coverage]:
        """
        gets a specific Coverage

        Parameters
        ----------
        id : CoverageId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[Coverage]
        """
        _response = self._client_wrapper.httpx_client.request(
            f"coverages/v1/{jsonable_encoder(id)}",
            base_url=self._client_wrapper.get_environment().pre_encounter,
            method="GET",
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                Coverage,
                parse_obj_as(
                    type_=Coverage,  # type: ignore
                    object_=_response_json,
                ),
            )
            return HttpResponse(response=_response, data=_data)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get_history(
        self,
        id: CoverageId,
        *,
        start: typing.Optional[dt.date] = None,
        end: typing.Optional[dt.date] = None,
        non_auto_updated_coverages_only: typing.Optional[bool] = None,
        sort_direction: typing.Optional[SortDirection] = None,
        limit: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[typing.List[Coverage]]:
        """
        Gets a coverage's history. Full history is returned if no filters are
        defined. The return list is ordered by version, defaulting to ascending.

        Parameters
        ----------
        id : CoverageId

        start : typing.Optional[dt.date]

        end : typing.Optional[dt.date]

        non_auto_updated_coverages_only : typing.Optional[bool]
            If true, only returns coverages that have NOT been auto-updated by the system.

        sort_direction : typing.Optional[SortDirection]
            Defaults to ascending. Sorts by version.

        limit : typing.Optional[int]
            Must be between 0 and 1000. No default.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[typing.List[Coverage]]
        """
        _response = self._client_wrapper.httpx_client.request(
            f"coverages/v1/{jsonable_encoder(id)}/history",
            base_url=self._client_wrapper.get_environment().pre_encounter,
            method="GET",
            params={
                "start": str(start) if start is not None else None,
                "end": str(end) if end is not None else None,
                "non_auto_updated_coverages_only": non_auto_updated_coverages_only,
                "sort_direction": sort_direction,
                "limit": limit,
            },
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                typing.List[Coverage],
                parse_obj_as(
                    type_=typing.List[Coverage],  # type: ignore
                    object_=_response_json,
                ),
            )
            return HttpResponse(response=_response, data=_data)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "BadRequestError":
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorBase4Xx,
                        parse_obj_as(
                            type_=ErrorBase4Xx,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "NotFoundError":
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorBase4Xx,
                        parse_obj_as(
                            type_=ErrorBase4Xx,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get_multi(
        self, *, patient_id: typing.Optional[str] = None, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[typing.List[Coverage]]:
        """
        Returns a list of Coverages based on the search criteria.

        Parameters
        ----------
        patient_id : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[typing.List[Coverage]]
        """
        _response = self._client_wrapper.httpx_client.request(
            "coverages/v1",
            base_url=self._client_wrapper.get_environment().pre_encounter,
            method="GET",
            params={
                "patient_id": patient_id,
            },
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                typing.List[Coverage],
                parse_obj_as(
                    type_=typing.List[Coverage],  # type: ignore
                    object_=_response_json,
                ),
            )
            return HttpResponse(response=_response, data=_data)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def scan(
        self, *, since: dt.datetime, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[typing.List[Coverage]]:
        """
        Scans up to 100 coverage updates. The since query parameter is inclusive, and the result list is ordered by updatedAt ascending.

        **Polling Pattern:**
        To continuously poll for updates without gaps:
        1. Make your initial request with a `since` timestamp (e.g., `since=2020-01-01T13:00:00.000Z`)
        2. The API returns up to 100 coverage records, sorted by `updated_at` ascending
        3. Find the `updated_at` value from the last record in the response
        4. Use that `updated_at` value as the `since` parameter in your next request
        5. Repeat steps 2-4 to ingest updates until you receive an empty list

        **Important Notes:**
        - The `since` parameter is inclusive, so you may receive the last record from the previous batch again (you can deduplicate by ID and version)
        - All coverage records include `updated_at`, `id`, `version`, `deactivated`, and `updating_user` fields for tracking changes
        - Timestamps have millisecond resolution for precise ordering

        Parameters
        ----------
        since : dt.datetime

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[typing.List[Coverage]]
        """
        _response = self._client_wrapper.httpx_client.request(
            "coverages/v1/updates/scan",
            base_url=self._client_wrapper.get_environment().pre_encounter,
            method="GET",
            params={
                "since": serialize_datetime(since),
            },
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                typing.List[Coverage],
                parse_obj_as(
                    type_=typing.List[Coverage],  # type: ignore
                    object_=_response_json,
                ),
            )
            return HttpResponse(response=_response, data=_data)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def batch_update_ppg(
        self,
        ppg_id: PayerPlanGroupId,
        *,
        request: PayerPlanGroupFields,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[None]:
        """
        Finds all coverages associated with the given ppg_id and updates the ppg_fields for each coverage.

        Parameters
        ----------
        ppg_id : PayerPlanGroupId

        request : PayerPlanGroupFields

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[None]
        """
        _response = self._client_wrapper.httpx_client.request(
            f"coverages/v1/batch-update-ppg/{jsonable_encoder(ppg_id)}",
            base_url=self._client_wrapper.get_environment().pre_encounter,
            method="POST",
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        if 200 <= _response.status_code < 300:
            return HttpResponse(response=_response, data=None)
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def check_eligibility(
        self,
        id: CoverageId,
        *,
        service_code: ServiceTypeCode,
        date_of_service: dt.date,
        npi: str,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[EligibilityCheckMetadata]:
        """
        Initiates an eligibility check. Returns the metadata of the check if successfully initiated.

        Parameters
        ----------
        id : CoverageId

        service_code : ServiceTypeCode

        date_of_service : dt.date

        npi : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[EligibilityCheckMetadata]
        """
        _response = self._client_wrapper.httpx_client.request(
            f"coverages/v1/{jsonable_encoder(id)}/eligibility",
            base_url=self._client_wrapper.get_environment().pre_encounter,
            method="POST",
            json={
                "service_code": service_code,
                "date_of_service": date_of_service,
                "npi": npi,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                EligibilityCheckMetadata,
                parse_obj_as(
                    type_=EligibilityCheckMetadata,  # type: ignore
                    object_=_response_json,
                ),
            )
            return HttpResponse(response=_response, data=_data)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get_eligibility(
        self, id: CoverageId, check_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[CoverageEligibilityCheckResponse]:
        """
        Gets the eligibility of a patient for a specific coverage if successful.

        Parameters
        ----------
        id : CoverageId

        check_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[CoverageEligibilityCheckResponse]
        """
        _response = self._client_wrapper.httpx_client.request(
            f"coverages/v1/{jsonable_encoder(id)}/eligibility/{jsonable_encoder(check_id)}",
            base_url=self._client_wrapper.get_environment().pre_encounter,
            method="GET",
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                CoverageEligibilityCheckResponse,
                parse_obj_as(
                    type_=CoverageEligibilityCheckResponse,  # type: ignore
                    object_=_response_json,
                ),
            )
            return HttpResponse(response=_response, data=_data)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawV1Client:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def create(
        self, *, request: MutableCoverage, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[Coverage]:
        """
        Creates a new Coverage. A Coverage provides the high-level identifiers and descriptors of a specific insurance plan for a specific individual - typically the information you can find on an insurance card. Additionally a coverage will include detailed benefits information covered by the specific plan for the individual.

        Parameters
        ----------
        request : MutableCoverage

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[Coverage]
        """
        _response = await self._client_wrapper.httpx_client.request(
            "coverages/v1",
            base_url=self._client_wrapper.get_environment().pre_encounter,
            method="POST",
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                Coverage,
                parse_obj_as(
                    type_=Coverage,  # type: ignore
                    object_=_response_json,
                ),
            )
            return AsyncHttpResponse(response=_response, data=_data)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def update(
        self,
        id: CoverageId,
        version: str,
        *,
        request: MutableCoverage,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[Coverage]:
        """
        Updates a Coverage. The path must contain the next version number to prevent race conditions. For example, if the current version of the coverage is n, you will need to send a request to this endpoint with `/{id}/n+1` to update the coverage. Updating historic versions is not supported.

        Parameters
        ----------
        id : CoverageId

        version : str

        request : MutableCoverage

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[Coverage]
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"coverages/v1/{jsonable_encoder(id)}/{jsonable_encoder(version)}",
            base_url=self._client_wrapper.get_environment().pre_encounter,
            method="PUT",
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                Coverage,
                parse_obj_as(
                    type_=Coverage,  # type: ignore
                    object_=_response_json,
                ),
            )
            return AsyncHttpResponse(response=_response, data=_data)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "NotFoundError":
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorBase4Xx,
                        parse_obj_as(
                            type_=ErrorBase4Xx,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "VersionConflictError":
                raise VersionConflictError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        VersionConflictErrorBody,
                        parse_obj_as(
                            type_=VersionConflictErrorBody,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get_multi_paginated(
        self,
        *,
        patient_id: typing.Optional[str] = None,
        payer_plan_group_id: typing.Optional[str] = None,
        page_token: typing.Optional[PageToken] = None,
        limit: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[CoveragesPage]:
        """
        Returns a page of Coverages based on the search criteria.

        Parameters
        ----------
        patient_id : typing.Optional[str]

        payer_plan_group_id : typing.Optional[str]

        page_token : typing.Optional[PageToken]

        limit : typing.Optional[int]
            Must be between 0 and 1000. Defaults to 100

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[CoveragesPage]
        """
        _response = await self._client_wrapper.httpx_client.request(
            "coverages/v1/get-multi-paginated",
            base_url=self._client_wrapper.get_environment().pre_encounter,
            method="GET",
            params={
                "patient_id": patient_id,
                "payer_plan_group_id": payer_plan_group_id,
                "page_token": page_token,
                "limit": limit,
            },
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                CoveragesPage,
                parse_obj_as(
                    type_=CoveragesPage,  # type: ignore
                    object_=_response_json,
                ),
            )
            return AsyncHttpResponse(response=_response, data=_data)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "BadRequestError":
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorBase4Xx,
                        parse_obj_as(
                            type_=ErrorBase4Xx,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get(
        self, id: CoverageId, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[Coverage]:
        """
        gets a specific Coverage

        Parameters
        ----------
        id : CoverageId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[Coverage]
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"coverages/v1/{jsonable_encoder(id)}",
            base_url=self._client_wrapper.get_environment().pre_encounter,
            method="GET",
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                Coverage,
                parse_obj_as(
                    type_=Coverage,  # type: ignore
                    object_=_response_json,
                ),
            )
            return AsyncHttpResponse(response=_response, data=_data)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get_history(
        self,
        id: CoverageId,
        *,
        start: typing.Optional[dt.date] = None,
        end: typing.Optional[dt.date] = None,
        non_auto_updated_coverages_only: typing.Optional[bool] = None,
        sort_direction: typing.Optional[SortDirection] = None,
        limit: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[typing.List[Coverage]]:
        """
        Gets a coverage's history. Full history is returned if no filters are
        defined. The return list is ordered by version, defaulting to ascending.

        Parameters
        ----------
        id : CoverageId

        start : typing.Optional[dt.date]

        end : typing.Optional[dt.date]

        non_auto_updated_coverages_only : typing.Optional[bool]
            If true, only returns coverages that have NOT been auto-updated by the system.

        sort_direction : typing.Optional[SortDirection]
            Defaults to ascending. Sorts by version.

        limit : typing.Optional[int]
            Must be between 0 and 1000. No default.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[typing.List[Coverage]]
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"coverages/v1/{jsonable_encoder(id)}/history",
            base_url=self._client_wrapper.get_environment().pre_encounter,
            method="GET",
            params={
                "start": str(start) if start is not None else None,
                "end": str(end) if end is not None else None,
                "non_auto_updated_coverages_only": non_auto_updated_coverages_only,
                "sort_direction": sort_direction,
                "limit": limit,
            },
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                typing.List[Coverage],
                parse_obj_as(
                    type_=typing.List[Coverage],  # type: ignore
                    object_=_response_json,
                ),
            )
            return AsyncHttpResponse(response=_response, data=_data)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "BadRequestError":
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorBase4Xx,
                        parse_obj_as(
                            type_=ErrorBase4Xx,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "NotFoundError":
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorBase4Xx,
                        parse_obj_as(
                            type_=ErrorBase4Xx,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get_multi(
        self, *, patient_id: typing.Optional[str] = None, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[typing.List[Coverage]]:
        """
        Returns a list of Coverages based on the search criteria.

        Parameters
        ----------
        patient_id : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[typing.List[Coverage]]
        """
        _response = await self._client_wrapper.httpx_client.request(
            "coverages/v1",
            base_url=self._client_wrapper.get_environment().pre_encounter,
            method="GET",
            params={
                "patient_id": patient_id,
            },
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                typing.List[Coverage],
                parse_obj_as(
                    type_=typing.List[Coverage],  # type: ignore
                    object_=_response_json,
                ),
            )
            return AsyncHttpResponse(response=_response, data=_data)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def scan(
        self, *, since: dt.datetime, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[typing.List[Coverage]]:
        """
        Scans up to 100 coverage updates. The since query parameter is inclusive, and the result list is ordered by updatedAt ascending.

        **Polling Pattern:**
        To continuously poll for updates without gaps:
        1. Make your initial request with a `since` timestamp (e.g., `since=2020-01-01T13:00:00.000Z`)
        2. The API returns up to 100 coverage records, sorted by `updated_at` ascending
        3. Find the `updated_at` value from the last record in the response
        4. Use that `updated_at` value as the `since` parameter in your next request
        5. Repeat steps 2-4 to ingest updates until you receive an empty list

        **Important Notes:**
        - The `since` parameter is inclusive, so you may receive the last record from the previous batch again (you can deduplicate by ID and version)
        - All coverage records include `updated_at`, `id`, `version`, `deactivated`, and `updating_user` fields for tracking changes
        - Timestamps have millisecond resolution for precise ordering

        Parameters
        ----------
        since : dt.datetime

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[typing.List[Coverage]]
        """
        _response = await self._client_wrapper.httpx_client.request(
            "coverages/v1/updates/scan",
            base_url=self._client_wrapper.get_environment().pre_encounter,
            method="GET",
            params={
                "since": serialize_datetime(since),
            },
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                typing.List[Coverage],
                parse_obj_as(
                    type_=typing.List[Coverage],  # type: ignore
                    object_=_response_json,
                ),
            )
            return AsyncHttpResponse(response=_response, data=_data)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def batch_update_ppg(
        self,
        ppg_id: PayerPlanGroupId,
        *,
        request: PayerPlanGroupFields,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[None]:
        """
        Finds all coverages associated with the given ppg_id and updates the ppg_fields for each coverage.

        Parameters
        ----------
        ppg_id : PayerPlanGroupId

        request : PayerPlanGroupFields

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[None]
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"coverages/v1/batch-update-ppg/{jsonable_encoder(ppg_id)}",
            base_url=self._client_wrapper.get_environment().pre_encounter,
            method="POST",
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        if 200 <= _response.status_code < 300:
            return AsyncHttpResponse(response=_response, data=None)
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def check_eligibility(
        self,
        id: CoverageId,
        *,
        service_code: ServiceTypeCode,
        date_of_service: dt.date,
        npi: str,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[EligibilityCheckMetadata]:
        """
        Initiates an eligibility check. Returns the metadata of the check if successfully initiated.

        Parameters
        ----------
        id : CoverageId

        service_code : ServiceTypeCode

        date_of_service : dt.date

        npi : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[EligibilityCheckMetadata]
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"coverages/v1/{jsonable_encoder(id)}/eligibility",
            base_url=self._client_wrapper.get_environment().pre_encounter,
            method="POST",
            json={
                "service_code": service_code,
                "date_of_service": date_of_service,
                "npi": npi,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                EligibilityCheckMetadata,
                parse_obj_as(
                    type_=EligibilityCheckMetadata,  # type: ignore
                    object_=_response_json,
                ),
            )
            return AsyncHttpResponse(response=_response, data=_data)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get_eligibility(
        self, id: CoverageId, check_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[CoverageEligibilityCheckResponse]:
        """
        Gets the eligibility of a patient for a specific coverage if successful.

        Parameters
        ----------
        id : CoverageId

        check_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[CoverageEligibilityCheckResponse]
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"coverages/v1/{jsonable_encoder(id)}/eligibility/{jsonable_encoder(check_id)}",
            base_url=self._client_wrapper.get_environment().pre_encounter,
            method="GET",
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                CoverageEligibilityCheckResponse,
                parse_obj_as(
                    type_=CoverageEligibilityCheckResponse,  # type: ignore
                    object_=_response_json,
                ),
            )
            return AsyncHttpResponse(response=_response, data=_data)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

import typing
from importlib import import_module

if typing.TYPE_CHECKING:
    from .types import (
        AdjustmentId,
        AppointmentId,
        BadRequestErrorMessage,
        BillingProviderCommercialLicenseType,
        ChargeCaptureClaimCreationId,
        ChargeCaptureId,
        ChargeCapturePostBilledChangeId,
        ClaimAdjustmentGroupCodes,
        ClaimId,
        ClaimSubmissionPayerResponsibilityType,
        Clearinghouse,
        ClinicalTrialId,
        Date,
        DateRangeOptionalEnd,
        Decimal,
        DelayReasonCode,
        Email,
        EmrPayerCrosswalk,
        EncounterExternalId,
        EncounterId,
        EntityConflictErrorMessage,
        EntityNotFoundErrorMessage,
        EpsdtReferralConditionIndicatorCode,
        ErrorMessage,
        FacilityTypeCode,
        InsuranceTypeCode,
        IntendedSubmissionMedium,
        InternalErrorMessage,
        InvoiceId,
        LinkUrl,
        NetworkType,
        NextResponsibleParty,
        Npi,
        OrganizationId,
        OrganizationNotAuthorizedErrorMessage,
        PageToken,
        PatientExternalId,
        PatientRelationshipToInsuredCodeAll,
        PayerPlanGroupId,
        PhoneNumber,
        PhoneNumberOptional,
        PhoneNumberType,
        PreEncounterAppointmentId,
        PreEncounterPatientId,
        Primitive,
        ProcedureModifier,
        ProviderCredentialingSpanId,
        ProviderId,
        QualifierCode,
        RateId,
        RegionNational,
        RegionStates,
        Regions,
        Regions_National,
        Regions_States,
        RemovableDateRangeOptionalEnd,
        RemovableDateRangeOptionalEnd_DateRange,
        RemovableDateRangeOptionalEnd_Remove,
        ReportTransmissionCode,
        ReportTypeCode,
        RequestValidationError,
        ResourcePage,
        SchemaId,
        ServiceLineId,
        ServiceLineUnits,
        SortDirection,
        SourceOfPaymentCode,
        State,
        StreetAddressBase,
        StreetAddressLongZip,
        StreetAddressLongZipOptional,
        StreetAddressOptionalBase,
        StreetAddressShortZip,
        StreetAddressShortZipOptional,
        TaskAssignmentId,
        TaskId,
        TaskNoteId,
        UnauthorizedErrorMessage,
        UnprocessableEntityErrorMessage,
        UpdatesDisabledDueToExternalSystemIntegrationErrorMessage,
        UserId,
        WorkQueueId,
    )
    from .errors import (
        BadRequestError,
        EntityConflictError,
        EntityNotFoundError,
        HttpRequestValidationError,
        HttpRequestValidationsError,
        InternalError,
        OrganizationNotAuthorizedError,
        UnauthorizedError,
        UnprocessableEntityError,
        UpdatesDisabledDueToExternalSystemIntegrationError,
    )
_dynamic_imports: typing.Dict[str, str] = {
    "AdjustmentId": ".types",
    "AppointmentId": ".types",
    "BadRequestError": ".errors",
    "BadRequestErrorMessage": ".types",
    "BillingProviderCommercialLicenseType": ".types",
    "ChargeCaptureClaimCreationId": ".types",
    "ChargeCaptureId": ".types",
    "ChargeCapturePostBilledChangeId": ".types",
    "ClaimAdjustmentGroupCodes": ".types",
    "ClaimId": ".types",
    "ClaimSubmissionPayerResponsibilityType": ".types",
    "Clearinghouse": ".types",
    "ClinicalTrialId": ".types",
    "Date": ".types",
    "DateRangeOptionalEnd": ".types",
    "Decimal": ".types",
    "DelayReasonCode": ".types",
    "Email": ".types",
    "EmrPayerCrosswalk": ".types",
    "EncounterExternalId": ".types",
    "EncounterId": ".types",
    "EntityConflictError": ".errors",
    "EntityConflictErrorMessage": ".types",
    "EntityNotFoundError": ".errors",
    "EntityNotFoundErrorMessage": ".types",
    "EpsdtReferralConditionIndicatorCode": ".types",
    "ErrorMessage": ".types",
    "FacilityTypeCode": ".types",
    "HttpRequestValidationError": ".errors",
    "HttpRequestValidationsError": ".errors",
    "InsuranceTypeCode": ".types",
    "IntendedSubmissionMedium": ".types",
    "InternalError": ".errors",
    "InternalErrorMessage": ".types",
    "InvoiceId": ".types",
    "LinkUrl": ".types",
    "NetworkType": ".types",
    "NextResponsibleParty": ".types",
    "Npi": ".types",
    "OrganizationId": ".types",
    "OrganizationNotAuthorizedError": ".errors",
    "OrganizationNotAuthorizedErrorMessage": ".types",
    "PageToken": ".types",
    "PatientExternalId": ".types",
    "PatientRelationshipToInsuredCodeAll": ".types",
    "PayerPlanGroupId": ".types",
    "PhoneNumber": ".types",
    "PhoneNumberOptional": ".types",
    "PhoneNumberType": ".types",
    "PreEncounterAppointmentId": ".types",
    "PreEncounterPatientId": ".types",
    "Primitive": ".types",
    "ProcedureModifier": ".types",
    "ProviderCredentialingSpanId": ".types",
    "ProviderId": ".types",
    "QualifierCode": ".types",
    "RateId": ".types",
    "RegionNational": ".types",
    "RegionStates": ".types",
    "Regions": ".types",
    "Regions_National": ".types",
    "Regions_States": ".types",
    "RemovableDateRangeOptionalEnd": ".types",
    "RemovableDateRangeOptionalEnd_DateRange": ".types",
    "RemovableDateRangeOptionalEnd_Remove": ".types",
    "ReportTransmissionCode": ".types",
    "ReportTypeCode": ".types",
    "RequestValidationError": ".types",
    "ResourcePage": ".types",
    "SchemaId": ".types",
    "ServiceLineId": ".types",
    "ServiceLineUnits": ".types",
    "SortDirection": ".types",
    "SourceOfPaymentCode": ".types",
    "State": ".types",
    "StreetAddressBase": ".types",
    "StreetAddressLongZip": ".types",
    "StreetAddressLongZipOptional": ".types",
    "StreetAddressOptionalBase": ".types",
    "StreetAddressShortZip": ".types",
    "StreetAddressShortZipOptional": ".types",
    "TaskAssignmentId": ".types",
    "TaskId": ".types",
    "TaskNoteId": ".types",
    "UnauthorizedError": ".errors",
    "UnauthorizedErrorMessage": ".types",
    "UnprocessableEntityError": ".errors",
    "UnprocessableEntityErrorMessage": ".types",
    "UpdatesDisabledDueToExternalSystemIntegrationError": ".errors",
    "UpdatesDisabledDueToExternalSystemIntegrationErrorMessage": ".types",
    "UserId": ".types",
    "WorkQueueId": ".types",
}


def __getattr__(attr_name: str) -> typing.Any:
    module_name = _dynamic_imports.get(attr_name)
    if module_name is None:
        raise AttributeError(f"No {attr_name} found in _dynamic_imports for module name -> {__name__}")
    try:
        module = import_module(module_name, __package__)
        if module_name == f".{attr_name}":
            return module
        else:
            return getattr(module, attr_name)
    except ImportError as e:
        raise ImportError(f"Failed to import {attr_name} from {module_name}: {e}") from e
    except AttributeError as e:
        raise AttributeError(f"Failed to get {attr_name} from {module_name}: {e}") from e


def __dir__():
    lazy_attrs = list(_dynamic_imports.keys())
    return sorted(lazy_attrs)


__all__ = [
    "AdjustmentId",
    "AppointmentId",
    "BadRequestError",
    "BadRequestErrorMessage",
    "BillingProviderCommercialLicenseType",
    "ChargeCaptureClaimCreationId",
    "ChargeCaptureId",
    "ChargeCapturePostBilledChangeId",
    "ClaimAdjustmentGroupCodes",
    "ClaimId",
    "ClaimSubmissionPayerResponsibilityType",
    "Clearinghouse",
    "ClinicalTrialId",
    "Date",
    "DateRangeOptionalEnd",
    "Decimal",
    "DelayReasonCode",
    "Email",
    "EmrPayerCrosswalk",
    "EncounterExternalId",
    "EncounterId",
    "EntityConflictError",
    "EntityConflictErrorMessage",
    "EntityNotFoundError",
    "EntityNotFoundErrorMessage",
    "EpsdtReferralConditionIndicatorCode",
    "ErrorMessage",
    "FacilityTypeCode",
    "HttpRequestValidationError",
    "HttpRequestValidationsError",
    "InsuranceTypeCode",
    "IntendedSubmissionMedium",
    "InternalError",
    "InternalErrorMessage",
    "InvoiceId",
    "LinkUrl",
    "NetworkType",
    "NextResponsibleParty",
    "Npi",
    "OrganizationId",
    "OrganizationNotAuthorizedError",
    "OrganizationNotAuthorizedErrorMessage",
    "PageToken",
    "PatientExternalId",
    "PatientRelationshipToInsuredCodeAll",
    "PayerPlanGroupId",
    "PhoneNumber",
    "PhoneNumberOptional",
    "PhoneNumberType",
    "PreEncounterAppointmentId",
    "PreEncounterPatientId",
    "Primitive",
    "ProcedureModifier",
    "ProviderCredentialingSpanId",
    "ProviderId",
    "QualifierCode",
    "RateId",
    "RegionNational",
    "RegionStates",
    "Regions",
    "Regions_National",
    "Regions_States",
    "RemovableDateRangeOptionalEnd",
    "RemovableDateRangeOptionalEnd_DateRange",
    "RemovableDateRangeOptionalEnd_Remove",
    "ReportTransmissionCode",
    "ReportTypeCode",
    "RequestValidationError",
    "ResourcePage",
    "SchemaId",
    "ServiceLineId",
    "ServiceLineUnits",
    "SortDirection",
    "SourceOfPaymentCode",
    "State",
    "StreetAddressBase",
    "StreetAddressLongZip",
    "StreetAddressLongZipOptional",
    "StreetAddressOptionalBase",
    "StreetAddressShortZip",
    "StreetAddressShortZipOptional",
    "TaskAssignmentId",
    "TaskId",
    "TaskNoteId",
    "UnauthorizedError",
    "UnauthorizedErrorMessage",
    "UnprocessableEntityError",
    "UnprocessableEntityErrorMessage",
    "UpdatesDisabledDueToExternalSystemIntegrationError",
    "UpdatesDisabledDueToExternalSystemIntegrationErrorMessage",
    "UserId",
    "WorkQueueId",
]



# This file was auto-generated by Fern from our API Definition.

import typing

from ......core import enum

T_Result = typing.TypeVar("T_Result")


class InvoiceSortField(enum.StrEnum):
    CREATED_AT = "CREATED_AT"
    UPDATED_AT = "UPDATED_AT"
    PATIENT_EXTERNAL_ID = "PATIENT_EXTERNAL_ID"
    NOTE = "NOTE"
    DUE_DATE = "DUE_DATE"
    STATUS = "STATUS"
    _UNKNOWN = "__INVOICESORTFIELD_UNKNOWN__"
    """
    This member is used for forward compatibility. If the value is not recognized by the enum, it will be stored here, and the raw value is accessible through `.value`.
    """

    @classmethod
    def _missing_(cls, value: typing.Any) -> "InvoiceSortField":
        unknown = cls._UNKNOWN
        unknown._value_ = value
        return unknown

    def visit(
        self,
        created_at: typing.Callable[[], T_Result],
        updated_at: typing.Callable[[], T_Result],
        patient_external_id: typing.Callable[[], T_Result],
        note: typing.Callable[[], T_Result],
        due_date: typing.Callable[[], T_Result],
        status: typing.Callable[[], T_Result],
        _unknown_member: typing.Callable[[str], T_Result],
    ) -> T_Result:
        if self is InvoiceSortField.CREATED_AT:
            return created_at()
        if self is InvoiceSortField.UPDATED_AT:
            return updated_at()
        if self is InvoiceSortField.PATIENT_EXTERNAL_ID:
            return patient_external_id()
        if self is InvoiceSortField.NOTE:
            return note()
        if self is InvoiceSortField.DUE_DATE:
            return due_date()
        if self is InvoiceSortField.STATUS:
            return status()
        return _unknown_member(self._value_)



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

import typing
from importlib import import_module

if typing.TYPE_CHECKING:
    from .types import (
        DenialReasonContent,
        DrugIdentification,
        DrugIdentificationOptional,
        MeasurementUnitCode,
        ServiceIdQualifier,
        ServiceLine,
        ServiceLineAdjustment,
        ServiceLineCreate,
        ServiceLineCreateBase,
        ServiceLineCreateBaseBase,
        ServiceLineCreateOptional,
        ServiceLineCreateStandalone,
        ServiceLineCreateStandaloneBase,
        ServiceLineCreateStandaloneBaseBase,
        ServiceLineDenialReason,
        ServiceLineEraData,
        ServiceLineUpdate,
        ServiceLineUpdateBase,
        TestResult,
        TestResultOptional,
        TestResultType,
        UniversalServiceLineCreate,
        UniversalServiceLineCreateStandalone,
        UniversalServiceLineUpdate,
    )
_dynamic_imports: typing.Dict[str, str] = {
    "DenialReasonContent": ".types",
    "DrugIdentification": ".types",
    "DrugIdentificationOptional": ".types",
    "MeasurementUnitCode": ".types",
    "ServiceIdQualifier": ".types",
    "ServiceLine": ".types",
    "ServiceLineAdjustment": ".types",
    "ServiceLineCreate": ".types",
    "ServiceLineCreateBase": ".types",
    "ServiceLineCreateBaseBase": ".types",
    "ServiceLineCreateOptional": ".types",
    "ServiceLineCreateStandalone": ".types",
    "ServiceLineCreateStandaloneBase": ".types",
    "ServiceLineCreateStandaloneBaseBase": ".types",
    "ServiceLineDenialReason": ".types",
    "ServiceLineEraData": ".types",
    "ServiceLineUpdate": ".types",
    "ServiceLineUpdateBase": ".types",
    "TestResult": ".types",
    "TestResultOptional": ".types",
    "TestResultType": ".types",
    "UniversalServiceLineCreate": ".types",
    "UniversalServiceLineCreateStandalone": ".types",
    "UniversalServiceLineUpdate": ".types",
}


def __getattr__(attr_name: str) -> typing.Any:
    module_name = _dynamic_imports.get(attr_name)
    if module_name is None:
        raise AttributeError(f"No {attr_name} found in _dynamic_imports for module name -> {__name__}")
    try:
        module = import_module(module_name, __package__)
        if module_name == f".{attr_name}":
            return module
        else:
            return getattr(module, attr_name)
    except ImportError as e:
        raise ImportError(f"Failed to import {attr_name} from {module_name}: {e}") from e
    except AttributeError as e:
        raise AttributeError(f"Failed to get {attr_name} from {module_name}: {e}") from e


def __dir__():
    lazy_attrs = list(_dynamic_imports.keys())
    return sorted(lazy_attrs)


__all__ = [
    "DenialReasonContent",
    "DrugIdentification",
    "DrugIdentificationOptional",
    "MeasurementUnitCode",
    "ServiceIdQualifier",
    "ServiceLine",
    "ServiceLineAdjustment",
    "ServiceLineCreate",
    "ServiceLineCreateBase",
    "ServiceLineCreateBaseBase",
    "ServiceLineCreateOptional",
    "ServiceLineCreateStandalone",
    "ServiceLineCreateStandaloneBase",
    "ServiceLineCreateStandaloneBaseBase",
    "ServiceLineDenialReason",
    "ServiceLineEraData",
    "ServiceLineUpdate",
    "ServiceLineUpdateBase",
    "TestResult",
    "TestResultOptional",
    "TestResultType",
    "UniversalServiceLineCreate",
    "UniversalServiceLineCreateStandalone",
    "UniversalServiceLineUpdate",
]



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
from ......core.pydantic_utilities import IS_PYDANTIC_V2
from .....commons.types.encounter_id import EncounterId
from .....users.resources.v_2.types.user_v_2 import UserV2
from .base_attachment import BaseAttachment
from .encounter_attachment_type import EncounterAttachmentType


class EncounterAttachment(BaseAttachment):
    encounter_id: EncounterId
    attachment_type: EncounterAttachmentType
    created_at: dt.datetime
    created_by: UserV2

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
from ........core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .....coverages.resources.v_1.types.member_info import MemberInfo
from .encounter import Encounter
from .provider import Provider


class EligibilityRequest(UniversalBaseModel):
    """
    An object representing the data for an eligibility request.
    """

    submitter_transaction_identifier: typing.Optional[str] = pydantic.Field(default=None)
    """
    A unique identifier for the eligibility check within the batch. Candid returns this identifier in the response for the
    /batch/{batch_id} polling endpoint so you can correlate benefit responses with the original eligibility check.
    """

    payer_id: str = pydantic.Field()
    """
    Supported payer ID values can be found [here](https://www.stedi.com/healthcare/network).
    """

    provider: Provider
    subscriber: MemberInfo = pydantic.Field()
    """
    The primary policyholder for the insurance plan or a dependent with a unique member ID. <Note>If a dependent has a unique member ID, include their information here and leave dependent undefined.</Note>
    """

    dependent: typing.Optional[MemberInfo] = pydantic.Field(default=None)
    """
    <Note>If a dependent has a unique member ID, include their information as subscriber and leave this field undefined.</Note>
    """

    encounter: typing.Optional[Encounter] = None
    get_existing_check_initiated_after: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    If not provided, this endpoint will run a fresh eligibility check. If provided, it will return an existing successful eligibility check if one exists that was initiated after provided date with the same parameters (Date of Service, Payer ID, Provider, Subscriber, Dependent, and Encounter).
    """

    source: typing.Optional[str] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from ......core import enum

T_Result = typing.TypeVar("T_Result")


class OtherDiagnosisInformationCodeQualifier(enum.StrEnum):
    CBF = "BF"
    """
    International Classification of Diseases Clinical Modification (ICD-9-CM) Diagnosis
    """

    CABF = "ABF"
    """
    International Classification of Diseases Clinical Modification (ICD-10-CM) Diagnosis
    """

    _UNKNOWN = "__OTHERDIAGNOSISINFORMATIONCODEQUALIFIER_UNKNOWN__"
    """
    This member is used for forward compatibility. If the value is not recognized by the enum, it will be stored here, and the raw value is accessible through `.value`.
    """

    @classmethod
    def _missing_(cls, value: typing.Any) -> "OtherDiagnosisInformationCodeQualifier":
        unknown = cls._UNKNOWN
        unknown._value_ = value
        return unknown

    def visit(
        self,
        cbf: typing.Callable[[], T_Result],
        cabf: typing.Callable[[], T_Result],
        _unknown_member: typing.Callable[[str], T_Result],
    ) -> T_Result:
        if self is OtherDiagnosisInformationCodeQualifier.CBF:
            return cbf()
        if self is OtherDiagnosisInformationCodeQualifier.CABF:
            return cabf()
        return _unknown_member(self._value_)



# This file was auto-generated by Fern from our API Definition.

import typing

from ......core import enum

T_Result = typing.TypeVar("T_Result")


class PointOfOriginForAdmissionOrVisitCode(enum.StrEnum):
    C_1 = "1"
    """
    NON-HEALTH CARE FACILITY POINT OF ORIGIN - Inpatient: The patient was admitted to this facility. Outpatient: The patient presented for outpatient services. Examples: Includes patients coming from home or workplace and patients receiving care at home (such as home health services).
    """

    C_2 = "2"
    """
    CLINIC OR PHYSICIAN'S OFFICE - Inpatient: The patient was admitted to this facility. Outpatient: The patient presented to this facility for outpatient services.
    """

    C_4 = "4"
    """
    TRANSFER FROM A HOSPITAL (DIFFERENT FACILITY) - Inpatient: The patient was admitted to this facility as a hospital transfer from an acute care facility where he or she was an inpatient or outpatient. Outpatient: The patient was transferred to this facility as an outpatient from an acute care facility. Excludes Transfers from Hospital Inpatient in the Same Facility (See Code D).
    """

    C_5 = "5"
    """
    TRANSFER FROM A SKILLED NURSING FACILITY (SNF), ASSISTED LIVING FACILITY (ALF), INTERMEDIATE CARE FACILITY (ICF) OR OTHER NURSING FACILITY (NF). - Inpatient: The patient was admitted to this facility as a transfer from a SNF, ALF, ICF or other NF where he or she was a resident. Outpatient: The patient presented to this facility for outpatient or referenced diagnostic services from a SNF, ALF, ICF or other NF where he or she was a resident.
    """

    C_6 = "6"
    """
    TRANSFER FROM ANOTHER HEALTH CARE FACILITY - Inpatient: The patient was admitted to this facility as a transfer from another type of health care facility not defined elsewhere in this code list. Outpatient: The patient presented to this facility for services from another health care facility not defined elsewhere in this code list.
    """

    C_8 = "8"
    """
    COURT/LAW ENFORCEMENT - Inpatient: The patient was admitted to this facility upon the direction of a court of law, or upon the request of a law enforcement agency representative. Outpatient: The patient presented to this facility upon the direction of a court of law, or upon the request of a law enforcement agency representative for outpatient or referenced diagnostic services. Usage Note: Includes transfers from incarceration facilities.
    """

    C_9 = "9"
    """
    INFORMATION NOT AVAILABLE - Inpatient: The patient's Point of Origin is not known. Outpatient: The patient's Point of Origin is not known.
    """

    CD = "D"
    """
    TRANSFER FROM ONE DISTINCT UNIT OF THE HOSPITAL TO ANOTHER DISTINCT UNIT OF THE SAME HOSPITAL RESULTING IN A SEPARATE CLAIM TO THE PAYER - Inpatient: The patient was admitted to this facility as a transfer from hospital inpatient within this hospital resulting in a separate claim to the payer. Outpatient: The patient received outpatient services in this facility as a transfer from within this hospital resulting in a separate claim to the payer. Usage Note: For purposes of this code, "Distinct Unit" is defined as a unique unit or level of care at the hospital requiring the issuance of a separate claim to the payer. Examples could include observation services, psychiatric units, rehabilitation units, a unit in a critical access hospital, or a swing bed located in an acute hospital.
    """

    CE = "E"
    """
    TRANSFER FROM AMBULATORY SURGERY CENTER - Inpatient: The patient was admitted to this facility as a transfer from an ambulatory surgery center. Outpatient: The patient presented to this facility for outpatient or referenced diagnostic services from an ambulatory surgery center.
    """

    CF = "F"
    """
    TRANSFER FROM A HOSPICE FACILITY - Inpatient: The patient was admitted to this facility as a transfer from a hospice facility. Outpatient: The patient presented to this facility for outpatient or referenced diagnostic services from a hospice facility.
    """

    CG = "G"
    """
    TRANSFER FROM A DESIGNATED DISASTER ALTERNATE CARE SITE (EFFECTIVE 7/1/20) - The patient was transferred to this facility from a Designated Disaster Alternate Care Site for inpatient or outpatient services.
    """

    _UNKNOWN = "__POINTOFORIGINFORADMISSIONORVISITCODE_UNKNOWN__"
    """
    This member is used for forward compatibility. If the value is not recognized by the enum, it will be stored here, and the raw value is accessible through `.value`.
    """

    @classmethod
    def _missing_(cls, value: typing.Any) -> "PointOfOriginForAdmissionOrVisitCode":
        unknown = cls._UNKNOWN
        unknown._value_ = value
        return unknown

    def visit(
        self,
        c_1: typing.Callable[[], T_Result],
        c_2: typing.Callable[[], T_Result],
        c_4: typing.Callable[[], T_Result],
        c_5: typing.Callable[[], T_Result],
        c_6: typing.Callable[[], T_Result],
        c_8: typing.Callable[[], T_Result],
        c_9: typing.Callable[[], T_Result],
        cd: typing.Callable[[], T_Result],
        ce: typing.Callable[[], T_Result],
        cf: typing.Callable[[], T_Result],
        cg: typing.Callable[[], T_Result],
        _unknown_member: typing.Callable[[str], T_Result],
    ) -> T_Result:
        if self is PointOfOriginForAdmissionOrVisitCode.C_1:
            return c_1()
        if self is PointOfOriginForAdmissionOrVisitCode.C_2:
            return c_2()
        if self is PointOfOriginForAdmissionOrVisitCode.C_4:
            return c_4()
        if self is PointOfOriginForAdmissionOrVisitCode.C_5:
            return c_5()
        if self is PointOfOriginForAdmissionOrVisitCode.C_6:
            return c_6()
        if self is PointOfOriginForAdmissionOrVisitCode.C_8:
            return c_8()
        if self is PointOfOriginForAdmissionOrVisitCode.C_9:
            return c_9()
        if self is PointOfOriginForAdmissionOrVisitCode.CD:
            return cd()
        if self is PointOfOriginForAdmissionOrVisitCode.CE:
            return ce()
        if self is PointOfOriginForAdmissionOrVisitCode.CF:
            return cf()
        if self is PointOfOriginForAdmissionOrVisitCode.CG:
            return cg()
        return _unknown_member(self._value_)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ......core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .license_type import LicenseType
from .organization_provider_address import OrganizationProviderAddress
from .provider_type import ProviderType


class OrganizationProviderBase(UniversalBaseModel):
    npi: str = pydantic.Field()
    """
    The NPI of the provider. This must be all digits [0-9] and exactly 10 characters long.
    """

    is_rendering: bool = pydantic.Field()
    """
    Whether the provider can be used to render services.
    """

    is_billing: bool = pydantic.Field()
    """
    Whether the provider can be used to bill services.
    """

    first_name: typing.Optional[str] = pydantic.Field(default=None)
    """
    The first name of the provider, if the provider is an individual.
    """

    last_name: typing.Optional[str] = pydantic.Field(default=None)
    """
    The last name of the provider, if the provider is an individual.
    """

    organization_name: typing.Optional[str] = pydantic.Field(default=None)
    """
    The name of the provider, if the provider is an organization.
    """

    provider_type: ProviderType = pydantic.Field()
    """
    Whether the provider is an individual (NPPES Type 1) or organization (NPPES Type 2) provider.
    """

    tax_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    If the provider has a contract with insurance, this must be the same tax ID given to the payer on an IRS W-9 form completed during contracting.
    """

    taxonomy_code: typing.Optional[str] = pydantic.Field(default=None)
    """
    A code designating classification and specialization.
    """

    license_type: LicenseType = pydantic.Field()
    """
    The type of license that the provider holds.
    """

    ptan: typing.Optional[str] = pydantic.Field(default=None)
    """
    The Provider Transaction Access Number for the provider.
    """

    medicaid_provider_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The medicaid provider ID for the provider.
    """

    addresses: typing.Optional[typing.List[OrganizationProviderAddress]] = pydantic.Field(default=None)
    """
    The addresses associated with this provider.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ......core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .....commons.types.date import Date
from ...v_2.types.authorized_signatory_update import AuthorizedSignatoryUpdate
from ...v_2.types.contract_status import ContractStatus
from ...v_2.types.date_update import DateUpdate
from ...v_2.types.insurance_types import InsuranceTypes
from ...v_2.types.regions_update import RegionsUpdate


class ContractUpdate(UniversalBaseModel):
    effective_date: typing.Optional[Date] = pydantic.Field(default=None)
    """
    The starting day upon which the contract is effective
    """

    expiration_date: typing.Optional[DateUpdate] = pydantic.Field(default=None)
    """
    An optional end day upon which the contract expires
    """

    regions: typing.Optional[RegionsUpdate] = pydantic.Field(default=None)
    """
    If present, the contract's rendering providers will be patched to this exact
    value, overriding what was set before.
    """

    contract_status: typing.Optional[ContractStatus] = None
    authorized_signatory: typing.Optional[AuthorizedSignatoryUpdate] = None
    commercial_insurance_types: typing.Optional[InsuranceTypes] = None
    medicare_insurance_types: typing.Optional[InsuranceTypes] = None
    medicaid_insurance_types: typing.Optional[InsuranceTypes] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from .....core.api_error import ApiError
from .....core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from .....core.http_response import AsyncHttpResponse, HttpResponse
from .....core.jsonable_encoder import jsonable_encoder
from .....core.pydantic_utilities import parse_obj_as
from .....core.request_options import RequestOptions
from ....commons.errors.entity_not_found_error import EntityNotFoundError
from ....commons.types.encounter_id import EncounterId
from ....commons.types.entity_not_found_error_message import EntityNotFoundErrorMessage
from .types.create_supplemental_information_request import CreateSupplementalInformationRequest
from .types.supplemental_information import SupplementalInformation
from .types.supplemental_information_id import SupplementalInformationId
from .types.update_supplemental_information_request import UpdateSupplementalInformationRequest

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawV1Client:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def get(
        self, encounter_id: EncounterId, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[typing.List[SupplementalInformation]]:
        """
        Parameters
        ----------
        encounter_id : EncounterId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[typing.List[SupplementalInformation]]
        """
        _response = self._client_wrapper.httpx_client.request(
            f"api/encounter-supplemental-information/v1/{jsonable_encoder(encounter_id)}",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="GET",
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                typing.List[SupplementalInformation],
                parse_obj_as(
                    type_=typing.List[SupplementalInformation],  # type: ignore
                    object_=_response_json,
                ),
            )
            return HttpResponse(response=_response, data=_data)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def create(
        self,
        encounter_id: EncounterId,
        *,
        request: CreateSupplementalInformationRequest,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[SupplementalInformation]:
        """
        Parameters
        ----------
        encounter_id : EncounterId

        request : CreateSupplementalInformationRequest

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[SupplementalInformation]
        """
        _response = self._client_wrapper.httpx_client.request(
            f"api/encounter-supplemental-information/v1/{jsonable_encoder(encounter_id)}",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="PUT",
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                SupplementalInformation,
                parse_obj_as(
                    type_=SupplementalInformation,  # type: ignore
                    object_=_response_json,
                ),
            )
            return HttpResponse(response=_response, data=_data)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "EntityNotFoundError":
                raise EntityNotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        EntityNotFoundErrorMessage,
                        parse_obj_as(
                            type_=EntityNotFoundErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def update(
        self,
        encounter_id: EncounterId,
        supplemental_information_id: SupplementalInformationId,
        *,
        request: UpdateSupplementalInformationRequest,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[SupplementalInformation]:
        """
        Parameters
        ----------
        encounter_id : EncounterId

        supplemental_information_id : SupplementalInformationId

        request : UpdateSupplementalInformationRequest

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[SupplementalInformation]
        """
        _response = self._client_wrapper.httpx_client.request(
            f"api/encounter-supplemental-information/v1/{jsonable_encoder(encounter_id)}/{jsonable_encoder(supplemental_information_id)}",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="PATCH",
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                SupplementalInformation,
                parse_obj_as(
                    type_=SupplementalInformation,  # type: ignore
                    object_=_response_json,
                ),
            )
            return HttpResponse(response=_response, data=_data)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "EntityNotFoundError":
                raise EntityNotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        EntityNotFoundErrorMessage,
                        parse_obj_as(
                            type_=EntityNotFoundErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def delete(
        self,
        encounter_id: EncounterId,
        supplemental_information_id: SupplementalInformationId,
        *,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[None]:
        """
        Parameters
        ----------
        encounter_id : EncounterId

        supplemental_information_id : SupplementalInformationId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[None]
        """
        _response = self._client_wrapper.httpx_client.request(
            f"api/encounter-supplemental-information/v1/{jsonable_encoder(encounter_id)}/{jsonable_encoder(supplemental_information_id)}",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="DELETE",
            request_options=request_options,
        )
        if 200 <= _response.status_code < 300:
            return HttpResponse(response=_response, data=None)
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "EntityNotFoundError":
                raise EntityNotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        EntityNotFoundErrorMessage,
                        parse_obj_as(
                            type_=EntityNotFoundErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawV1Client:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def get(
        self, encounter_id: EncounterId, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[typing.List[SupplementalInformation]]:
        """
        Parameters
        ----------
        encounter_id : EncounterId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[typing.List[SupplementalInformation]]
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"api/encounter-supplemental-information/v1/{jsonable_encoder(encounter_id)}",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="GET",
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                typing.List[SupplementalInformation],
                parse_obj_as(
                    type_=typing.List[SupplementalInformation],  # type: ignore
                    object_=_response_json,
                ),
            )
            return AsyncHttpResponse(response=_response, data=_data)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def create(
        self,
        encounter_id: EncounterId,
        *,
        request: CreateSupplementalInformationRequest,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[SupplementalInformation]:
        """
        Parameters
        ----------
        encounter_id : EncounterId

        request : CreateSupplementalInformationRequest

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[SupplementalInformation]
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"api/encounter-supplemental-information/v1/{jsonable_encoder(encounter_id)}",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="PUT",
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                SupplementalInformation,
                parse_obj_as(
                    type_=SupplementalInformation,  # type: ignore
                    object_=_response_json,
                ),
            )
            return AsyncHttpResponse(response=_response, data=_data)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "EntityNotFoundError":
                raise EntityNotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        EntityNotFoundErrorMessage,
                        parse_obj_as(
                            type_=EntityNotFoundErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def update(
        self,
        encounter_id: EncounterId,
        supplemental_information_id: SupplementalInformationId,
        *,
        request: UpdateSupplementalInformationRequest,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[SupplementalInformation]:
        """
        Parameters
        ----------
        encounter_id : EncounterId

        supplemental_information_id : SupplementalInformationId

        request : UpdateSupplementalInformationRequest

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[SupplementalInformation]
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"api/encounter-supplemental-information/v1/{jsonable_encoder(encounter_id)}/{jsonable_encoder(supplemental_information_id)}",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="PATCH",
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                SupplementalInformation,
                parse_obj_as(
                    type_=SupplementalInformation,  # type: ignore
                    object_=_response_json,
                ),
            )
            return AsyncHttpResponse(response=_response, data=_data)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "EntityNotFoundError":
                raise EntityNotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        EntityNotFoundErrorMessage,
                        parse_obj_as(
                            type_=EntityNotFoundErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def delete(
        self,
        encounter_id: EncounterId,
        supplemental_information_id: SupplementalInformationId,
        *,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[None]:
        """
        Parameters
        ----------
        encounter_id : EncounterId

        supplemental_information_id : SupplementalInformationId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[None]
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"api/encounter-supplemental-information/v1/{jsonable_encoder(encounter_id)}/{jsonable_encoder(supplemental_information_id)}",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="DELETE",
            request_options=request_options,
        )
        if 200 <= _response.status_code < 300:
            return AsyncHttpResponse(response=_response, data=None)
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "EntityNotFoundError":
                raise EntityNotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        EntityNotFoundErrorMessage,
                        parse_obj_as(
                            type_=EntityNotFoundErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

from ....core import enum

T_Result = typing.TypeVar("T_Result")


class NetworkType(enum.StrEnum):
    PPO = "12"
    """
    Preferred Provider Organization (PPO)
    """

    POS = "13"
    """
    Point of Service (POS)
    """

    EPO = "14"
    """
    Exclusive Provider Organization (EPO)
    """

    INDEMNITY_INSURANCE = "15"
    """
    Indemnity Insurance
    """

    HMO_MEDICARE_RISK = "16"
    """
    Health Maintenance Organization (HMO) Medicare Risk
    """

    DMO = "17"
    """
    Dental Maintenance Organization
    """

    AUTO = "AM"
    """
    Automobile Medical
    """

    CHAMPUS = "CH"
    """
    CHAMPUS
    """

    DISABILITY = "DS"
    """
    Disability
    """

    HMO = "HM"
    """
    Health Maintenance Organization (HMO)
    """

    LIABILITY = "LM"
    """
    Liability Medical
    """

    MEDICARE_PART_A = "MA"
    """
    Medicare Part A
    """

    MEDICARE_PART_B = "MB"
    """
    Medicare Part B
    """

    MEDICAID = "MC"
    """
    Medicaid
    """

    OTHER_FEDERAL_PROGRAM = "OF"
    """
    Other Federal Program
    """

    TITLE_V = "TV"
    """
    Title V
    """

    VETERANS_AFFAIRS_PLAN = "VA"
    """
    Veterans Affairs Plan
    """

    WORKERS_COMP_HEALTH_CLAIM = "WC"
    """
    Workers' Compensation Health Claim
    """

    MUTUALLY_DEFINED = "ZZ"
    """
    Mutually Defined
    """

    COMMERCIAL_INSURANCE_CO = "CI"
    """
    Commercial Insurance
    """

    BLUE_CROSS_BLUE_SHIELD = "BL"
    """
    Blue Cross Blue Shield
    """

    _UNKNOWN = "__NETWORKTYPE_UNKNOWN__"
    """
    This member is used for forward compatibility. If the value is not recognized by the enum, it will be stored here, and the raw value is accessible through `.value`.
    """

    @classmethod
    def _missing_(cls, value: typing.Any) -> "NetworkType":
        unknown = cls._UNKNOWN
        unknown._value_ = value
        return unknown

    def visit(
        self,
        ppo: typing.Callable[[], T_Result],
        pos: typing.Callable[[], T_Result],
        epo: typing.Callable[[], T_Result],
        indemnity_insurance: typing.Callable[[], T_Result],
        hmo_medicare_risk: typing.Callable[[], T_Result],
        dmo: typing.Callable[[], T_Result],
        auto: typing.Callable[[], T_Result],
        champus: typing.Callable[[], T_Result],
        disability: typing.Callable[[], T_Result],
        hmo: typing.Callable[[], T_Result],
        liability: typing.Callable[[], T_Result],
        medicare_part_a: typing.Callable[[], T_Result],
        medicare_part_b: typing.Callable[[], T_Result],
        medicaid: typing.Callable[[], T_Result],
        other_federal_program: typing.Callable[[], T_Result],
        title_v: typing.Callable[[], T_Result],
        veterans_affairs_plan: typing.Callable[[], T_Result],
        workers_comp_health_claim: typing.Callable[[], T_Result],
        mutually_defined: typing.Callable[[], T_Result],
        commercial_insurance_co: typing.Callable[[], T_Result],
        blue_cross_blue_shield: typing.Callable[[], T_Result],
        _unknown_member: typing.Callable[[str], T_Result],
    ) -> T_Result:
        if self is NetworkType.PPO:
            return ppo()
        if self is NetworkType.POS:
            return pos()
        if self is NetworkType.EPO:
            return epo()
        if self is NetworkType.INDEMNITY_INSURANCE:
            return indemnity_insurance()
        if self is NetworkType.HMO_MEDICARE_RISK:
            return hmo_medicare_risk()
        if self is NetworkType.DMO:
            return dmo()
        if self is NetworkType.AUTO:
            return auto()
        if self is NetworkType.CHAMPUS:
            return champus()
        if self is NetworkType.DISABILITY:
            return disability()
        if self is NetworkType.HMO:
            return hmo()
        if self is NetworkType.LIABILITY:
            return liability()
        if self is NetworkType.MEDICARE_PART_A:
            return medicare_part_a()
        if self is NetworkType.MEDICARE_PART_B:
            return medicare_part_b()
        if self is NetworkType.MEDICAID:
            return medicaid()
        if self is NetworkType.OTHER_FEDERAL_PROGRAM:
            return other_federal_program()
        if self is NetworkType.TITLE_V:
            return title_v()
        if self is NetworkType.VETERANS_AFFAIRS_PLAN:
            return veterans_affairs_plan()
        if self is NetworkType.WORKERS_COMP_HEALTH_CLAIM:
            return workers_comp_health_claim()
        if self is NetworkType.MUTUALLY_DEFINED:
            return mutually_defined()
        if self is NetworkType.COMMERCIAL_INSURANCE_CO:
            return commercial_insurance_co()
        if self is NetworkType.BLUE_CROSS_BLUE_SHIELD:
            return blue_cross_blue_shield()
        return _unknown_member(self._value_)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ........core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .....common.types.additional_payer_information import AdditionalPayerInformation
from .....common.types.address import Address
from .....common.types.payer_id import PayerId
from .....common.types.payer_plan_group_id import PayerPlanGroupId
from .....common.types.period import Period
from .insurance_type_code import InsuranceTypeCode
from .network_type import NetworkType


class InsurancePlan(UniversalBaseModel):
    member_id: str
    payer_id: PayerId
    payer_name: str
    additional_payer_information: typing.Optional[AdditionalPayerInformation] = None
    group_number: typing.Optional[str] = None
    name: typing.Optional[str] = None
    plan_type: typing.Optional[NetworkType] = None
    type: typing.Optional[InsuranceTypeCode] = None
    period: typing.Optional[Period] = None
    insurance_card_image_locator: typing.Optional[str] = None
    address: typing.Optional[Address] = None
    payer_plan_group_id: typing.Optional[PayerPlanGroupId] = pydantic.Field(default=None)
    """
    The ID of the Candid configured payer plan group associated with this coverage
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from ......core import enum

T_Result = typing.TypeVar("T_Result")


class ServiceFacilityStatus(enum.StrEnum):
    """
    Examples
    --------
    from candid.resources.organization_service_facilities.resources.v_2 import (
        ServiceFacilityStatus,
    )

    ServiceFacilityStatus.ACTIVE
    """

    ACTIVE = "active"
    """
    The location is operational.
    """

    INACTIVE = "inactive"
    """
    The location is no longer used.
    """

    SUSPENDED = "suspended"
    """
    TThe location is temporarily closed.
    """

    _UNKNOWN = "__SERVICEFACILITYSTATUS_UNKNOWN__"
    """
    This member is used for forward compatibility. If the value is not recognized by the enum, it will be stored here, and the raw value is accessible through `.value`.
    """

    @classmethod
    def _missing_(cls, value: typing.Any) -> "ServiceFacilityStatus":
        unknown = cls._UNKNOWN
        unknown._value_ = value
        return unknown

    def visit(
        self,
        active: typing.Callable[[], T_Result],
        inactive: typing.Callable[[], T_Result],
        suspended: typing.Callable[[], T_Result],
        _unknown_member: typing.Callable[[str], T_Result],
    ) -> T_Result:
        if self is ServiceFacilityStatus.ACTIVE:
            return active()
        if self is ServiceFacilityStatus.INACTIVE:
            return inactive()
        if self is ServiceFacilityStatus.SUSPENDED:
            return suspended()
        return _unknown_member(self._value_)
